
import ddf.minim.*;
import ddf.minim.signals.*;
import ddf.minim.analysis.*;
import ddf.minim.effects.*;

import processing.opengl.*;
import com.modestmaps.*;
import com.modestmaps.core.*;
import com.modestmaps.geo.*;
import com.modestmaps.providers.*;


//
// This is a test of the interactive Modest Maps library for Processing
// You must have modestmaps in your libraries folder, see INSTALL for details
//

// this is the only bit that's needed to show a map:
InteractiveMap map;

// buttons take x,y and width,height:
ZoomButton out = new ZoomButton(5, 5, 14, 14, false);
ZoomButton in = new ZoomButton(22, 5, 14, 14, true);
PanButton up = new PanButton(14, 25, 14, 14, UP);
PanButton down = new PanButton(14, 57, 14, 14, DOWN);
PanButton left = new PanButton(5, 41, 14, 14, LEFT);
PanButton right = new PanButton(22, 41, 14, 14, RIGHT);

//XML
XML[] sound;

// all the buttons in one place, for looping:
Button[] buttons = { 
  in, out, up, down, left, right
};

Circle[] circles = new Circle[150];

PFont font;

boolean gui = true;

double tx, ty, sc;

XML[] pos; 
Minim minim;
AudioPlayer[] player2;

Location[] mylocation;
  String tokens[] = null;
  Location location2;
  Point2f pp;
   float lonAux;
  Location locationAux;
  Point2f pAux;


void setup() {
  size(640*1, 480*1, OPENGL);
  smooth();

  minim = new Minim(this);
  player2=new AudioPlayer[100];
  // create a new map, optionally specify a provider

  // OpenStreetMap would be like this:
  map = new InteractiveMap(this, new OpenStreetMapProvider());
  // but it's a free open source project, so don't bother their server too much

  // AOL/MapQuest provides open tiles too
  // see http://developer.mapquest.com/web/products/open/map for terms
  // and this is how to use them:
  String template = "http://{S}.mqcdn.com/tiles/1.0.0/osm/{Z}/{X}/{Y}.png";
  String[] subdomains = new String[] { 
    "otile1", "otile2", "otile3", "otile4"
  }; // optional
  //map = new InteractiveMap(this, new TemplatedMapProvider(template, subdomains));

  // others would be "new Microsoft.HybridProvider()" or "new Microsoft.AerialProvider()"
  // the Google ones get blocked after a few hundred tiles
  // the Yahoo ones look terrible because they're not 256px squares :)

  

  // set a default font for labels
  font = createFont("Helvetica", 12);

  // enable the mouse wheel, for zooming
  addMouseWheelListener(new java.awt.event.MouseWheelListener() { 
    public void mouseWheelMoved(java.awt.event.MouseWheelEvent evt) { 
      mouseWheel(evt.getWheelRotation());
    }
  }
  ); 

  //explore the rss and parse it
  XML rss = loadXML("soundscape.rss");
  // Get all  elements
  XML[] title = rss.getChildren("channel/item/title");
  for (int i = 0; i < title.length; i++) {
    println("title" + " " + title[i].getContent());
  }
  XML[] atributes = rss.getChildren("channel/item/notours:soundpoint/notours:attributes");
  for (int i = 0; i < atributes.length; i++) {
    println("atributes"+ " " + atributes[i].getContent());
  }
  pos = rss.getChildren("channel/item/georss:circle");
  for (int i = 0; i < pos.length; i++) {
    println("GPS" + " " + pos[i].getContent());
  }
  tokens=pos[0].getContent().split(" ");
  location2 = new Location(Float.valueOf(tokens[0].trim()).floatValue(), Float.valueOf(tokens[1].trim()).floatValue());
  // set the initial location and zoom level to Gijon:
  map.setCenterZoom(location2, 16);  
  // zoom 0 is the whole world, 19 is street level
  
  sound = rss.getChildren("channel/item/notours:soundpoint/notours:sound/notours:file");
  for (int i = 0; i < sound.length; i++) {
    println("sound" + " " + sound[i].getContent());
    //finally create the objects
    circles[i]=new Circle();
  }
  
  
}

void draw() {
  background(0);

  // draw the map:
  map.draw();
  // (that's it! really... everything else is interactions now)

  smooth();

  for (int i=0;i<pos.length;i++) {
    tokens=pos[i].getContent().split(" ");
    //circles[i]=new Circle(Float.valueOf(tokens[0].trim()).floatValue(), Float.valueOf(tokens[1].trim()).floatValue());
    //paint something to try
    location2 = new Location(Float.valueOf(tokens[0].trim()).floatValue(), Float.valueOf(tokens[1].trim()).floatValue());
    pp = map.locationPoint(location2);
    
    //abracadabra...
    lonAux=getLongAtDistance(Float.valueOf(tokens[0].trim()).floatValue(),Float.valueOf(tokens[1].trim()).floatValue(),Float.valueOf(tokens[2].trim()).floatValue());
    //println(lonAux);
  
    locationAux=new Location(Float.valueOf(tokens[0].trim()).floatValue(),lonAux);
    pAux=map.locationPoint(locationAux);
    
    //set Center and Radius in pixels and Audio File
    //circles[i]=new Circle(pp.x,pp.y,Math.abs((pp.x-pAux.x)*2), sound[i].getContent());
    circles[i].setCenterX(pp.x);
    circles[i].setCenterY(pp.y);
    circles[i].setCenterRadius(Math.abs((pp.x-pAux.x)*2));
    circles[i].setAudioPath(sound[i].getContent());
    
  }
  
  //draw all the circles every iteration
  for (int i=0;i<pos.length;i++) { 
    fill(0, 255, 128,70);
    stroke(255, 255, 0);
    ellipse(circles[i].centerX, circles[i].centerY, circles[i].centerRadius, circles[i].centerRadius);     
  }
  
  //control audio play 
  //1. detect if I am over the Circle
  //2. if over, see if it is already playing
  double d2;
  for (int i = 0; i < sound.length; i++) {
   
   //calculo distancia en pixeles
   d2=Math.sqrt(Math.pow(mouseX-circles[i].centerX,2)+Math.pow(mouseY-circles[i].centerY,2))*2;
   //println(d2);
   //println("**");
   //println(circles[i].centerRadius);
    if(d2<circles[i].centerRadius){
         //println("dentro");
         if(!circles[i].playing) { 
           
           player2[i] = minim.loadFile(circles[i].audioPath);
           circles[i].load(circles[i].audioPath);
           player2[i].loop();
           player2[i].play();
         } 
     } else {
         //println("fuera");
         if(circles[i].playing) {
           player2[i].pause(); 
           circles[i].stop();
         }
       }
  
  }
  


  // draw all the buttons and check for mouse-over
  boolean hand = false;
  if (gui) {
    for (int i = 0; i < buttons.length; i++) {
      buttons[i].draw();
      hand = hand || buttons[i].mouseOver();
    }
  }

  // if we're over a button, use the finger pointer
  // otherwise use the cross
  // (I wish Java had the open/closed hand for "move" cursors)
  cursor(hand ? HAND : CROSS);

  // see if the arrow keys or +/- keys are pressed:
  // (also check space and z, to reset or round zoom levels)
  if (keyPressed) {
    if (key == CODED) {
      if (keyCode == LEFT) {
        map.tx += 5.0/map.sc;
      }
      else if (keyCode == RIGHT) {
        map.tx -= 5.0/map.sc;
      }
      else if (keyCode == UP) {
        map.ty += 5.0/map.sc;
      }
      else if (keyCode == DOWN) {
        map.ty -= 5.0/map.sc;
      }
    }  
    else if (key == '+' || key == '=') {
      map.sc *= 1.05;
    }
    else if (key == '_' || key == '-' && map.sc > 2) {
      map.sc *= 1.0/1.05;
    }
  }

  if (gui) {
    textFont(font, 12);

    // grab the lat/lon location under the mouse point:
    Location location = map.pointLocation(mouseX, mouseY);

    // draw the mouse location, bottom left:
    fill(0);
    noStroke();
    rect(5, height-5-g.textSize, textWidth("mouse: " + location), g.textSize+textDescent());
    fill(255, 255, 0);
    textAlign(LEFT, BOTTOM);
    text("mouse: " + location, 5, height-5);

    // grab the center
    location = map.pointLocation(width/2, height/2);

    // draw the center location, bottom right:
    fill(0);
    noStroke();
    float rw = textWidth("map: " + location);
    rect(width-5-rw, height-5-g.textSize, rw, g.textSize+textDescent());
    fill(255, 255, 0);
    textAlign(RIGHT, BOTTOM);
    text("map: " + location, width-5, height-5);

    location = new Location(51.500, -0.126);
    Point2f p = map.locationPoint(location);

    fill(0, 255, 128);
    stroke(255, 255, 0);
    ellipse(p.x, p.y, 10, 10);
  }  

  //println((float)map.sc);
  //println((float)map.tx + " " + (float)map.ty);
  //println();
}

void keyReleased() {
  if (key == 'g' || key == 'G') {
    gui = !gui;
  }
  else if (key == 's' || key == 'S') {
    save("modest-maps-app.png");
  }
  else if (key == 'z' || key == 'Z') {
    map.sc = pow(2, map.getZoom());
  }
  else if (key == ' ') {
    map.sc = 2.0;
    map.tx = -128;
    map.ty = -128;
  }
}


// see if we're over any buttons, otherwise tell the map to drag
void mouseDragged() {
  boolean hand = false;
  if (gui) {
    for (int i = 0; i < buttons.length; i++) {
      hand = hand || buttons[i].mouseOver();
      if (hand) break;
    }
  }
  if (!hand) {
    map.mouseDragged();
  }
}

// zoom in or out:
void mouseWheel(int delta) {
  float sc = 1.0;
  if (delta < 0) {
    sc = 1.05;
  }
  else if (delta > 0) {
    sc = 1.0/1.05;
  }
  float mx = mouseX - width/2;
  float my = mouseY - height/2;
  map.tx -= mx/map.sc;
  map.ty -= my/map.sc;
  map.sc *= sc;
  map.tx += mx/map.sc;
  map.ty += my/map.sc;
}

// see if we're over any buttons, and respond accordingly:
void mouseClicked() {
  if (in.mouseOver()) {
    map.zoomIn();
  }
  else if (out.mouseOver()) {
    map.zoomOut();
  }
  else if (up.mouseOver()) {
    map.panUp();
  }
  else if (down.mouseOver()) {
    map.panDown();
  }
  else if (left.mouseOver()) {
    map.panLeft();
  }
  else if (right.mouseOver()) {
    map.panRight();
  }

}
  
  float getLongAtDistance(float latitude, float longitude, float distance){
    float R=6371000;
    double lo;
    double lat=Math.toRadians(latitude);
    double lng=Math.toRadians(longitude);  
    
    lo=lng + (Math.acos((Math.cos(distance/R)-Math.sin(lat)*Math.sin(lat))/(Math.cos(lat)*Math.cos(lat))));
    return (float)Math.toDegrees(lo);
}

void keyPressed() {
  if (key == 's' || key == 'S') {
      // Saves a TIFF file named "diagonal.tif"
      save("noTours.tif");
    }
}


